"""Test writing settings on SLZB devices."""

import urllib

from aiohttp import ClientSession
from aresponses import ResponsesMockServer
import pytest

from pysmlight import Api2
from pysmlight.const import Settings
from pysmlight.exceptions import SmlightAuthError, SmlightConnectionError, SmlightError

host = "slzb-06.local"


async def test_settings_toggle(aresponses: ResponsesMockServer) -> None:
    """Test toggling settings on SLZB devices."""
    aresponses.add(
        host,
        "/settings/saveParams",
        "POST",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text="ok",
        ),
    )
    async with ClientSession() as session:
        client = Api2(host, session=session)
        page, key = Settings.DISABLE_LEDS.value
        res = await client.set_toggle(page, key, True)
        assert res

        req = aresponses.history[0][0]
        assert req.content_type == "application/x-www-form-urlencoded"
        body = urllib.parse.parse_qs(await req.text())
        assert body
        assert int(body["pageId"][0]) == page.value
        assert bool(body["ha"][0]) is True
        value = True if body[key][0] in "on" else False
        assert value


@pytest.mark.parametrize(
    ("code", "exception"), [(401, SmlightAuthError), (404, SmlightConnectionError)]
)
async def test_settings_toggle_post_error(
    aresponses: ResponsesMockServer, code: int, exception: SmlightError
) -> None:
    """Test toggling settings on SLZB devices."""
    aresponses.add(
        host,
        "/settings/saveParams",
        "POST",
        aresponses.Response(
            status=code,
            headers={"Content-Type": "application/json"},
            text="ok",
        ),
    )
    async with ClientSession() as session:
        client = Api2(host, session=session)
        with pytest.raises(exception):
            page, key = Settings.DISABLE_LEDS.value
            await client.set_toggle(page, key, True)



"""Tests for authenticating with SLZB-06x devices."""

from unittest.mock import patch

from aiohttp import ClientSession
from aiohttp.client_exceptions import ClientConnectionError
from aresponses import ResponsesMockServer
import pytest

from pysmlight.exceptions import SmlightAuthError, SmlightConnectionError
from pysmlight.web import Api2, webClient

host = "slzb-06.local"
USER = "admin"
PASSWORD = "admin"


async def test_auth_not_needed(aresponses: ResponsesMockServer) -> None:
    """Test that authentication is not needed to access the API."""
    aresponses.add(
        host,
        "/api2",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
        ),
    )
    async with ClientSession() as session:
        client = Api2(host, session=session)
        assert await client.check_auth_needed() is False


async def test_auth_is_required(aresponses: ResponsesMockServer) -> None:
    """Test that authentication is required to access the API."""
    aresponses.add(
        host,
        "/api2",
        "GET",
        aresponses.Response(
            status=401,
            headers={"Content-Type": "application/json"},
            text="wrong login or password",
        ),
        repeat=2,
    )
    async with ClientSession() as session:
        client = Api2(host, session=session)
        assert await client.check_auth_needed() is True
        with pytest.raises(SmlightAuthError):
            await client.authenticate(USER, PASSWORD)


async def test_auth_is_success(aresponses: ResponsesMockServer) -> None:
    """Test that authentication has access the API."""
    aresponses.add(
        host,
        "/api2",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
        ),
    )
    async with ClientSession() as session:
        client = Api2(host, session=session)
        assert await client.authenticate(USER, PASSWORD) is True


async def test_with_async(aresponses: ResponsesMockServer) -> None:
    """Test async with usage of webClient."""
    host = "slzb-06p7.local"
    aresponses.add(
        host,
        "/api2",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
        ),
        repeat=2,
    )
    async with webClient("slzb-06.local") as client:
        assert client.session is not None
        client.set_host(host)
        assert client.host == host
        assert await client.check_auth_needed() is False


@pytest.mark.asyncio
@patch("aiohttp.ClientSession.get")
async def test_auth_connector_error(mock_get) -> None:
    """Test auth connect failed."""

    async def mock_raise(*args, **kwargs):
        raise ClientConnectionError("Mocked connection error")

    mock_get.return_value.__aenter__.side_effect = mock_raise
    async with ClientSession() as session:
        client = Api2(host, session=session)

        with pytest.raises(SmlightConnectionError):
            await client.check_auth_needed()



"""Test sending commands to SLZB devices."""

from __future__ import annotations

import logging

from aiohttp import ClientSession
from aresponses import ResponsesMockServer

from pysmlight import Api2

_LOGGER = logging.getLogger(__name__)

host = "slzb-06.local"


def aresponses_fixture(aresponses: ResponsesMockServer):
    aresponses.add(
        host,
        "/api2",
        "GET",
        aresponses.Response(
            status=200, headers={"Content-Type": "application/json"}, text="ok"
        ),
    )


async def test_cmds_zb_bsl(aresponses: ResponsesMockServer) -> None:
    """Test sending ZB bootloader command to SLZB devices."""

    aresponses_fixture(aresponses)

    async with ClientSession() as session:
        client = Api2(host, session=session)

        await client.cmds.zb_bootloader()

        data = aresponses.history[0].request.query
        expected = {"action": 4, "cmd": 2}
        for k, v in data.items():
            assert expected[k] == int(v)


async def test_cmds_core_reboot(aresponses: ResponsesMockServer) -> None:
    """Test sending ZB reset command to SLZB devices."""

    aresponses_fixture(aresponses)

    async with ClientSession() as session:
        client = Api2(host, session=session)

        await client.cmds.reboot()

        data = aresponses.history[0].request.query
        expected = {"action": 4, "cmd": 3}
        for k, v in data.items():
            assert expected[k] == int(v)


async def test_cmds_zb_reboot(aresponses: ResponsesMockServer) -> None:
    """Test sending ZB reset command to SLZB devices."""

    aresponses_fixture(aresponses)

    async with ClientSession() as session:
        client = Api2(host, session=session)

        await client.cmds.zb_restart()

        data = aresponses.history[0].request.query
        expected = {"action": 4, "cmd": 1}
        for k, v in data.items():
            assert expected[k] == int(v)


async def test_cmds_zb_reconnect(aresponses: ResponsesMockServer) -> None:
    """Test sending ZB router reconnect command to SLZB devices."""

    aresponses_fixture(aresponses)

    async with ClientSession() as session:
        client = Api2(host, session=session)

        await client.cmds.zb_router()

        data = aresponses.history[0].request.query
        expected = {"action": 4, "cmd": 0}
        for k, v in data.items():
            assert expected[k] == int(v)


async def test_cmds_zb_reconnect_idx(aresponses: ResponsesMockServer) -> None:
    """Test sending ZB router reconnect command to SLZB devices."""

    aresponses_fixture(aresponses)

    async with ClientSession() as session:
        client = Api2(host, session=session)

        await client.cmds.zb_router(idx=1)

        data = aresponses.history[0].request.query
        expected = {"action": 4, "cmd": 0, "idx": 1}
        for k, v in data.items():
            assert expected[k] == int(v)



import asyncio
from collections.abc import Callable
import json
import logging
from unittest.mock import AsyncMock, Mock, call, patch

from aiohttp import ClientSession, web
from aiohttp.client_exceptions import SocketTimeoutError
from aresponses import ResponsesMockServer

from pysmlight.const import Actions, Events, Settings
from pysmlight.models import SettingsEvent
from pysmlight.sse import MessageEvent, sseClient
from pysmlight.web import Api2

_LOGGER = logging.getLogger(__name__)

host = "slzb-06.local"


def prepare_events():
    mock_events = [
        {"event": "LOG_STR", "data": "ConfigHelper|write config", "id": None},
        {"event": "EVENT_INET_STATE", "data": "ok", "id": None},
        {
            "event": "SAVE_PARAMS",
            "data": '{"page":8,"origin":"ha","changes":{"disableLeds":true,'
            '"nightMode":false},"needReboot":false}',
            "id": None,
        },
        {
            "event": "API2_WIFISCANSTATUS",
            "data": (
                '{"wifi":[{"ssid":"WifiAAAA19","rssi":-90,"channel":11,' '"secure":3}]}'
            ),
            "id": None,
        },
    ]

    event_stream = []
    for event in mock_events:
        event_data = []
        for field, value in event.items():
            if value is not None:
                event_data.append(f"{field}: {value}".encode())
        event_data.append(b"\n")
        event_stream.append(b"\n".join(event_data))
    return event_stream


events = prepare_events()


async def mock_sse_stream(request):
    async def event_stream():
        for message in events:
            yield message
            await asyncio.sleep(0.2)

    stream = web.StreamResponse(
        status=200,
        reason="OK",
        headers={"Content-Type": "text/event-stream"},
    )
    await stream.prepare(request)
    async for chunk in event_stream():
        await stream.write(chunk)
    await stream.write_eof()
    await stream.close()
    return stream


async def test_sse_stream(aresponses: ResponsesMockServer) -> None:
    """Test sse stream handling."""
    unload: list[Callable] = []
    log_message_handler = Mock()
    all_message_handler = Mock()
    settings_message_handler = Mock()
    aresponses.add(f"{host}:81", "/", "GET", mock_sse_stream)
    async with ClientSession() as session:
        client = Api2(host, session=session)
        unload.append(client.sse.register_callback(Events.LOG_STR, log_message_handler))
        unload.append(
            client.sse.register_callback(Events.CATCH_ALL, all_message_handler)
        )
        unload.append(
            client.sse.register_settings_cb(
                Settings.DISABLE_LEDS, settings_message_handler
            )
        )

        def wifi_cb(event: MessageEvent):
            assert event.data
            assert event.type == "API2_WIFISCANSTATUS"
            assert json.loads(event.data)["wifi"]

        with patch("pysmlight.Api2.get", return_value=None):
            unload.append(await client.scan_wifi(wifi_cb))

        await client.sse.sse_stream()

        assert log_message_handler.call_count == 1
        assert all_message_handler.call_count == 4
        assert settings_message_handler.call_count == 1
        assert settings_message_handler.call_args == call(
            SettingsEvent(
                page=8, origin="ha", needReboot=False, setting={"disableLeds": True}
            )
        )

        # remove callbacks
        for remove_cb in unload:
            remove_cb()
        unload.clear()
        assert len(client.sse.callbacks) == 1


async def test_get_param_inetstate(aresponses: ResponsesMockServer) -> None:
    """Test command to trigger inetState event."""

    async def response_handler(request):
        params = request.query
        assert int(params["action"]) == Actions.API_GET_PARAM.value
        assert params["param"] == "inetState"
        return aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text="ok",
        )

    aresponses.add(
        host,
        "/api2",
        "GET",
        response_handler,
    )
    async with ClientSession() as session:
        client = Api2(host, session=session)
        assert await client.get_param("inetState") == "ok"
        # invalid param
        assert await client.get_param("inet") is None


async def test_sse_client_task():
    async with ClientSession() as session:
        client = sseClient(host, session)
        client.legacy_api = True

        with patch.object(
            client, "sse_stream", new_callable=AsyncMock
        ) as mock_sse_stream:
            mock_sse_stream.side_effect = SocketTimeoutError

            task = asyncio.create_task(client.client())
            assert task is not None
            assert not task.done()
            await asyncio.sleep(0.1)
            assert client.timeout.sock_read == 600
            client.sse_stream.assert_called_once()

            task.cancel()
            try:
                await task
            except asyncio.CancelledError:
                pass



"""Tests for retrieving firmware information for SLZB-06x devices."""

from aiohttp import ClientSession
from aresponses import ResponsesMockServer
import pytest

from pysmlight import Api2, Firmware
from pysmlight.const import Actions

from . import load_fixture

host = "slzb-06.local"

MOCK_FIRMWARE_ESP = Firmware(
    ver="v2.5.2",
    mode="ESP",
    link="https://localhost/firmware.bin",
    notes="CHANGELOG v2.5.2\r\nFixed bug with the lights\nMore fixes",
)

MOCK_FIRMWARE_ZB = Firmware(
    rev="20240315",
    mode="ZB",
    link="https://localhost/firmware.bin",
    baud=115200,
    type=0,
    prod=True,
    notes=None,
)


@pytest.mark.asyncio
async def test_esp_firmware_update_get(aresponses: ResponsesMockServer) -> None:
    def response_handler(request):
        params = request.query
        assert params["action"] == str(Actions.API_FLASH_ESP.value)
        assert params["fwUrl"]
        return aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text="ok",
        )

    aresponses.add(
        host,
        "/api2",
        "GET",
        response=response_handler,
    )
    async with ClientSession() as session:
        client = Api2(host, session=session)
        response = await client.fw_update(MOCK_FIRMWARE_ESP)
        assert response


@pytest.mark.asyncio
async def test_zb_firmware_update_get(aresponses: ResponsesMockServer) -> None:
    def response_handler(request):
        params = request.query
        assert params["action"] == str(Actions.API_FLASH_ZB.value)
        assert params["fwUrl"]
        assert params["fwVer"] == "20240315"
        assert params["baud"] == "115200"
        assert params["fwType"] == "0"
        assert params["fwCh"] == "0"
        return aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text="ok",
        )

    aresponses.add(
        host,
        "/api2",
        "GET",
        response=response_handler,
    )
    async with ClientSession() as session:
        client = Api2(host, session=session)
        response = await client.fw_update(MOCK_FIRMWARE_ZB)
        assert response


@pytest.mark.asyncio
async def test_zb_firmware_update_idx_get(aresponses: ResponsesMockServer) -> None:
    def response_handler(request):
        params = request.query
        assert params["action"] == str(Actions.API_FLASH_ZB.value)
        assert params["fwUrl"]
        assert params["fwVer"] == "20240315"
        assert params["baud"] == "115200"
        assert params["fwType"] == "0"
        assert params["fwCh"] == "0"
        assert params["zbChipIdx"] == "1"
        return aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text="ok",
        )

    aresponses.add(
        host,
        "/api2",
        "GET",
        response=response_handler,
    )
    # info = await client.get_info()
    async with ClientSession() as session:
        client = Api2(host, session=session)
        response = await client.fw_update(MOCK_FIRMWARE_ZB, idx=1)
        assert response


@pytest.mark.asyncio
async def test_zb_old_firmware_update_idx(aresponses: ResponsesMockServer) -> None:
    def response_handler(request):
        params = request.query
        assert params["action"] == str(Actions.API_FLASH_ZB.value)
        assert params["fwUrl"]
        assert params["fwVer"] == "20240315"
        assert params["baud"] == "115200"
        assert params["fwType"] == "0"
        assert params["fwCh"] == "0"
        assert params["zbChipNum"] == "5"
        return aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text="ok",
        )

    aresponses.add(
        host,
        "/api2",
        "GET",
        response=response_handler,
    )
    aresponses.add(
        host,
        "/ha_info",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text=load_fixture("slzb-06-info.json"),
        ),
    )

    async with ClientSession() as session:
        client = Api2(host, session=session)
        info = await client.get_info()
        assert info.sw_version == "v2.5.2"
        response = await client.fw_update(MOCK_FIRMWARE_ZB, idx=1)
        assert response


async def test_format_release_notes() -> None:
    """Test formatting release notes."""
    async with ClientSession() as session:
        client = Api2(host, session=session)
        firmware = MOCK_FIRMWARE_ESP
        formatted = client._format_notes(firmware)
        assert formatted
        assert (
            formatted
            == "CHANGELOG v2.5.2\n\n* Fixed bug with the lights\n* More fixes\n"
        )

        firmware = MOCK_FIRMWARE_ZB
        formatted = client._format_notes(firmware)
        assert formatted is None


async def test_info_get_firmware_zb(aresponses: ResponsesMockServer) -> None:
    aresponses.add(
        "updates.smlight.tech",
        "/services/api/slzb-06x-ota.php",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text=load_fixture("slzb-06-zb-fw.json"),
        ),
    )
    async with ClientSession() as session:
        client = Api2(host, session=session)
        fw = await client.get_firmware_version("dev", device="SLZB-06M", mode="zigbee")
        assert fw
        assert len(fw) == 5
        firmware = fw[0]
        assert firmware.link
        assert firmware.mode == "ZB"
        assert firmware.dev is True
        assert firmware.ver == "20240510"
        assert firmware.type == 0


async def test_info_get_firmware_zb_filtered(aresponses: ResponsesMockServer) -> None:
    aresponses.add(
        "updates.smlight.tech",
        "/services/api/slzb-06x-ota.php",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text=load_fixture("slzb-06-zb-fw.json"),
        ),
    )
    async with ClientSession() as session:
        client = Api2(host, session=session)
        fw = await client.get_firmware_version(
            "dev", device="SLZB-06M", mode="zigbee", zb_type=0
        )
        assert fw
        assert len(fw) == 3
        firmware = fw[0]
        assert firmware.link
        assert firmware.mode == "ZB"
        assert firmware.dev is True
        assert firmware.ver == "20240510"
        assert firmware.type == 0


async def test_info_get_firmware_zb2(aresponses: ResponsesMockServer) -> None:
    """Test getting ZB firmware information for second radio."""
    aresponses.add(
        "updates.smlight.tech",
        "/services/api/slzb-06x-ota.php",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text=load_fixture("slzb-06-zb2-fw.json"),
        ),
    )
    async with ClientSession() as session:
        client = Api2(host, session=session)
        fw = await client.get_firmware_version(
            "dev", device="SLZB-MR1", mode="zigbee", idx=1
        )
        assert fw
        assert len(fw) == 9
        firmware = fw[0]
        assert firmware.link
        assert firmware.mode == "ZB"
        assert firmware.dev is True
        assert firmware.ver == "20240716"
        assert firmware.type == 0


async def test_info_get_firmware_esp(aresponses: ResponsesMockServer) -> None:
    aresponses.add(
        "updates.smlight.tech",
        "/services/api/slzb-06x-ota.php",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text=load_fixture("slzb-06-esp-fw.json"),
        ),
    )
    async with ClientSession() as session:
        client = Api2(host, session=session)
        fw = await client.get_firmware_version("release", mode="esp")
        assert fw
        firmware = fw[0]
        assert len(firmware.link) > 20
        assert firmware.mode == "ESP"
        assert firmware.dev is False
        assert firmware.rev == "20240229"
        assert firmware.ver == "v2.0.18"
        assert firmware.type is None
        assert firmware.notes
        assert len(firmware.notes.split("\n")) == 5


async def test_info_get_firmware_espu(aresponses: ResponsesMockServer) -> None:
    aresponses.add(
        "updates.smlight.tech",
        "/services/api/slzb-06x-ota.php",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text=load_fixture("slzb-06U-esp-fw.json"),
        ),
    )
    async with ClientSession() as session:
        client = Api2(host, session=session)
        fw = await client.get_firmware_version(
            "release", device="SLZB-MR1U", mode="esp32"
        )
        assert fw
        firmware = fw[0]
        assert len(firmware.link) > 20
        assert firmware.mode == "ESPs3"
        assert firmware.dev is False
        assert firmware.rev == "20250829"
        assert firmware.ver == "v3.0.0"
        assert firmware.type is None
        assert firmware.notes
        assert len(firmware.notes.split("\n")) == 3


async def test_info_get_firmware_none(aresponses: ResponsesMockServer) -> None:
    aresponses.add(
        "updates.smlight.tech",
        "/services/api/slzb-06x-ota.php",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text="[]",
        ),
    )
    async with ClientSession() as session:
        client = Api2(host, session=session)
        fw = await client.get_firmware_version(
            "release", device="SLZB-TEST", mode="esp32"
        )
        assert fw is None


async def test_resolve_zigbee_device() -> None:
    """Test the _resolve_zigbee_device method mapping for devices."""
    async with ClientSession() as session:
        client = Api2(host, session=session)

        assert client._resolve_zigbee_device("SLZB-MR1", 0) == "SLZB-06M"
        assert client._resolve_zigbee_device("SLZB-MR1", 1) == "SLZB-06p7V2"

        assert client._resolve_zigbee_device("SLZB-MR3U", 0) == "SLZB-MR3U"
        assert client._resolve_zigbee_device("SLZB-MR3U", 1) == "SLZB-06p10"

        assert client._resolve_zigbee_device("SLZB-06p7U", 0) == "SLZB-06p7V2"

        # out of bounds index
        assert client._resolve_zigbee_device("SLZB-MR1", 2) == "SLZB-MR1"
        assert client._resolve_zigbee_device("SLZB-MR1", -1) == "SLZB-MR1"

        # no mapped values
        assert client._resolve_zigbee_device("SLZB-06P10", 0) == "SLZB-06P10"


async def test_mr3u_device_id() -> None:
    """Test that SLZB-MR3U subdevice lookup returns device ID 23."""
    from pysmlight.const import Devices

    assert Devices.get("SLZB-MR3U") == 23



"""Tests for retrieving device information from SLZB-06x devices."""

import json
from unittest.mock import patch

from aiohttp import ClientSession
from aiohttp.client_exceptions import ClientConnectionError
from aresponses import ResponsesMockServer
import pytest

from pysmlight import Api2, Info, Radio
from pysmlight.const import Settings
from pysmlight.exceptions import SmlightAuthError, SmlightConnectionError

from . import load_fixture

host = "slzb-06.local"


async def test_info_device_info(aresponses: ResponsesMockServer) -> None:
    """Test getting SLZB device information."""
    aresponses.add(
        host,
        "/ha_info",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text=load_fixture("slzb-06-info.json"),
        ),
    )
    async with ClientSession() as session:
        client = Api2(host, session=session)
        info: Info = await client.get_info()
        assert info

        assert info.wifi_mode == 0
        assert info.ram_total == 285
        assert info.fs_total == 3456
        assert info.zb_ram_size == 296
        assert info.zb_flash_size == 1024
        assert info.coord_mode == 0
        assert info.device_ip == "192.168.1.63"
        assert info.fw_channel == "dev"
        assert info.MAC == "DD:88:FC:AA:EE:FF"
        assert info.model == "SLZB-06p10"
        assert info.sw_version == "v2.5.2"
        assert info.zb_hw == "CC2674P10"
        assert info.zb_version == "20240315"
        assert info.zb_type == 0
        assert info.legacy_api == 0
        assert info.hostname == "SLZB-06P10"
        assert len(info.radios) == 1
        assert info.radios is not None
        assert info.radios[0] == Radio(
            chip_index=0,
            zb_channel=None,
            zb_flash_size=1024,
            zb_hw="CC2674P10",
            zb_ram_size=296,
            zb_version="20240315",
            zb_type=0,
            radioModes=None,
        )


async def test_info_device_mr_info(aresponses: ResponsesMockServer) -> None:
    """Test getting SLZB device information for multi radios."""
    aresponses.add(
        host,
        "/ha_info",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text=load_fixture("slzb-06-info-radios.json"),
        ),
    )
    async with ClientSession() as session:
        client = Api2(host, session=session)
        info: Info = await client.get_info()
        assert info

        assert info.wifi_mode == 0
        assert info.ram_total == 300
        assert info.fs_total == 3456
        assert info.coord_mode == 0
        assert info.device_ip == "192.168.1.62"
        assert info.fw_channel == "dev"
        assert info.MAC == "DD:88:FC:AA:EE:FF"
        assert info.model == "SLZB-MR1"
        assert info.sw_version == "v2.7.1"
        assert info.legacy_api == 0
        assert info.hostname == "SLZB-MR1"

        assert info.radios is not None
        assert len(info.radios) == 2
        assert info.radios[0] == Radio(
            chip_index=0,
            zb_channel=1,
            zb_flash_size=768,
            zb_hw="EFR32MG21",
            zb_ram_size=96,
            zb_version="20240510",
            zb_type=0,
            radioModes=[True, True, True, False, False],
        )
        assert info.radios[1] == Radio(
            chip_index=1,
            zb_channel=1,
            zb_flash_size=704,
            zb_hw="CC2652P7",
            zb_ram_size=152,
            zb_version="20240716",
            zb_type=0,
            radioModes=[True, True, True, False, False],
        )


async def test_info_get_auth_fail(aresponses: ResponsesMockServer) -> None:
    """Test getting SLZB device information."""
    aresponses.add(
        host,
        "/ha_info",
        "GET",
        aresponses.Response(
            status=401,
            headers={"Content-Type": "application/json"},
            text="wrong login or password",
        ),
    )
    async with ClientSession() as session:
        client = Api2(host, session=session)
        with pytest.raises(SmlightAuthError):
            await client.get_info()


@pytest.mark.asyncio
@patch("aiohttp.ClientSession.get")
async def test_info_get_connector_error(mock_get) -> None:
    """Test getting SLZB device information."""

    async def mock_raise(*args, **kwargs):
        raise ClientConnectionError("Mocked connection error")

    mock_get.return_value.__aenter__.side_effect = mock_raise
    async with ClientSession() as session:
        client = Api2(host, session=session)

        with pytest.raises(SmlightConnectionError):
            await client.get_info()
        with pytest.raises(SmlightConnectionError):
            await client.get_sensors()


@pytest.mark.asyncio
@patch("aiohttp.ClientSession.post")
async def test_info_post_connector_error(mock_post) -> None:
    """Test getting SLZB device information."""

    async def mock_raise(*args, **kwargs):
        raise ClientConnectionError("Mocked connection error")

    mock_post.return_value.__aenter__.side_effect = mock_raise
    async with ClientSession() as session:
        client = Api2(host, session=session)
        _page, _toggle = Settings.NIGHT_MODE.value
        with pytest.raises(SmlightConnectionError):
            await client.set_toggle(_page, _toggle, True)


async def test_info_sensors(aresponses: ResponsesMockServer) -> None:
    """Test getting SLZB sensor data."""
    aresponses.add(
        host,
        "/ha_sensors",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text=load_fixture("slzb-06-sensors.json"),
        ),
    )
    async with ClientSession() as session:
        client = Api2(host, session=session)
        sensors = await client.get_sensors()
        assert sensors

        assert sensors.esp32_temp == 39.44
        assert sensors.zb_temp == 35.76
        assert sensors.zb_temp2 == 34.20
        assert sensors.uptime == 700
        assert sensors.socket_uptime is None
        assert sensors.ram_usage == 91
        assert sensors.fs_used == 192
        assert sensors.ethernet is True
        assert sensors.wifi_connected is False
        assert sensors.wifi_status == 2
        assert sensors.disable_leds is False
        assert sensors.night_mode is False
        assert sensors.auto_zigbee is False


async def test_info_legacy_info(aresponses: ResponsesMockServer) -> None:
    """Test getting legacy device info from devices with old firmware."""
    headers = {
        "Content-Type": "application/json",
        "respValuesArr": json.dumps(json.loads(load_fixture("slzb-06-resparr.json"))),
    }
    aresponses.add(
        host,
        "/ha_info",
        "GET",
        aresponses.Response(
            status=404,
            headers={"Content-Type": "application/json"},
        ),
    )
    aresponses.add(
        host,
        "/api2",
        "GET",
        aresponses.Response(status=200, headers=headers, text="Some html"),
    )
    async with ClientSession() as session:
        client = Api2(host, session=session)
        info: Info = await client.get_info()
        assert info

        assert info.device_ip == "192.168.1.157"
        assert info.fw_channel == "release"
        assert info.MAC == "DD:88:FC:AA:EE:FF"
        assert info.model == "SLZB-06p10"
        assert info.sw_version == "v2.0.20"
        assert info.zb_hw == "CC2674P10"
        assert info.zb_version is None
        assert info.zb_channel == 2
        assert info.legacy_api == 1


async def test_info_legacy_info2(aresponses: ResponsesMockServer) -> None:
    """Test getting legacy device info from devices with old firmware."""
    headers = {
        "Content-Type": "application/json",
        "respValuesArr": json.dumps(
            json.loads(load_fixture("slzb-06-resparr-0.9.9.json"))
        ),
    }
    aresponses.add(
        host,
        "/ha_info",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text="URL NOT FOUND",
        ),
    )
    aresponses.add(
        host,
        "/api",
        "GET",
        aresponses.Response(status=200, headers=headers, text="Some html"),
    )
    async with ClientSession() as session:
        client = Api2(host, session=session)
        info: Info = await client.get_info()
        assert info

        assert info.device_ip == "192.168.1.157"
        assert info.MAC == "DD:88:FC:AA:EE:FF"
        assert info.model == "SLZB-06"
        assert info.sw_version == "0.9.9"
        assert info.legacy_api == 2


async def test_info_invalid_versions() -> None:
    """Test conversion of invalid firmware versions (per awesomeVersion)

    factory firmware that may have .plus suffix
    development firmware that may have .dev suffix."""
    info = Info(sw_version="v2.3.6.plus")
    assert info.sw_version == "v2.3.6.1"
    info = Info(sw_version="v2.3.6.plus2")
    assert info.sw_version == "v2.3.6.2"
    info = Info(sw_version="v2.8.0.dev")
    assert info.sw_version == "v2.8.0.dev0"
